#!/bin/sh
usage()
{
    cat <<EOF
Usage: docker-build.sh [OPTIONS]
Options:
        [--debian=distros]
        [--ubuntu=distros]
        [--verbose]
        [--upload]
EOF
    exit $1
}


centos_map()
{
    dist=$1
    case $dist in
	centos5)
	    F=5;;
	centos6)
	    F=6;;
	centos7)
	    F=7;;
	*)
	    echo "$dist not supported"
	    exit 1
    esac
    return $F
}

prepare_deb()
{
    TYPE=$1
    DISTS=$2

    for dist in ${DISTS}; do
	test -f build/$TYPE/$dist && sudo rm -fr build/$TYPE/$dist
	mkdir -p build/$TYPE
	cp -a deb-src build/$TYPE/$dist
	cp ${ID_DEB_BUILD}/in-docker-deb.sh build/$TYPE/$dist
    done
}

run_deb()
{
    TYPE=$1
    DISTS=$2

    for dist in ${DISTS}; do
	cd build/$TYPE/$dist
	echo "FROM $TYPE:$dist" >Dockerfile
	echo "ENV DEBIAN_FRONTEND noninteractive" >>Dockerfile
	echo 'RUN apt-get update && apt-get -q install -y wget devscripts equivs' >>Dockerfile
	echo 'RUN wget -q -O - http://ftp.indexdata.dk/debian/indexdata.asc|apt-key add -' >>Dockerfile
	echo "RUN echo \"deb http://ftp.indexdata.dk/$TYPE $dist main\" >/etc/apt/sources.list.d/indexdata.list" >>Dockerfile
	docker build -t indexdata-build-$TYPE:$dist .
	echo -n "$TYPE $dist "
	docker run --rm  -v "$PWD:/build" -w /build/$PRODUCT-$VERSION indexdata-build-$TYPE:$dist ../in-docker-deb.sh >log 2>&1
	if test $? -eq 0; then
		echo Done
	else
		echo "Fail. Inspect build/$TYPE/$dist/log"
		exit 1
	fi
	cd ../../..
    done
}

D0=`dirname $0`
ID_DEB_BUILD=`cd $D0; pwd`

UPLOAD_HOST=${UPLOAD_HOST:=ftp.indexdata.dk}
UPLOAD_PATH=${UPLOAD_PATH:=/home/ftp/pub}
upload=false
while test $# -gt 0; do
    case "$1" in
        -*=*) optarg=`echo "$1" | sed 's/[-_a-zA-Z0-9]*=//'` ;;
        *) optarg= ;;
    esac
    case $1 in
        --debian=*)
            DEBIAN_DIST_A=$optarg
            ;;
        --ubuntu=*)
            UBUNTU_DIST_A=$optarg
            ;;
        --centos=*)
            CENTOS_DIST_A=$optarg
            ;;
        --upload)
            upload=true
            ;;
        --verbose)
	    set -x
            ;;
        -*)
            do_help=yes
            ;;
    esac
    shift
done
if test -f IDMETA;  then
    . ./IDMETA
else
    echo "IDMETA missing"
    exit 1
fi
if test "$DEBIAN_DIST_A" -o "$UBUNTU_DIST_A" -o "$CENTOS_DIST_A"; then
    DEBIAN_DIST=$DEBIAN_DIST_A
    UBUNTU_DIST=$UBUNTU_DIST_A
    CENTOS_DIST=$CENTOS_DIST_A
fi

if test "$CENTOS_DIST"; then
    RNAME=`awk '/Name:/ {print $2}' *.spec`
    if test "$NAME" -a "$NAME" != "$RNAME"; then
	echo "NAME in IDMETA ($NAME) does not match RPM spec NAME ($RNAME)"
	exit 1
    fi
    PRODUCT=$RNAME
    centos=true
else
    centos=false
fi

if test "$DEBIAN_DIST" -o "$UBUNTU_DIST"; then
    debian=true
    DNAME=`awk '/Source:/ {print $2}' debian/control`
    if test "$NAME" -a "$NAME" != "$DNAME"; then
        echo "NAME in IDMETA ($NAME) does not match debian/control ($DNAME)"
        exit 1
    fi
    PRODUCT=$DNAME
else
    debian=false
fi

TAR=${PRODUCT}-${VERSION}.tar.gz
if test ! -f "$TAR"; then
    echo "tar $TAR missing"
    exit 1
fi

if $upload; then
    UPLOAD_URI=${UPLOAD_HOST}:${UPLOAD_PATH}/${PRODUCT}
    echo "Uploading to ${UPLOAD_URI}"
    if $centos; then
	for dist in ${CENTOS_DIST}; do
	    centos_map $dist
	    F=$?
	    test $F = "5" && F=5.5
	    ssh ${UPLOAD_HOST} "mkdir -p ${UPLOAD_PATH}/${PRODUCT}/redhat/centos/$F/RPMS"
	    ssh ${UPLOAD_HOST} "mkdir -p ${UPLOAD_PATH}/${PRODUCT}/redhat/centos/$F/SRPMS"
	    scp -r build/centos/$dist/rpmbuild/RPMS ${UPLOAD_URI}/redhat/centos/$F
	    scp -r build/centos/$dist/rpmbuild/SRPMS ${UPLOAD_URI}/redhat/centos/$F
	done
	if ! ssh ${UPLOAD_HOST} "sudo -n /home/indexdata/proj/git-tools/update-archive/update-yum-archive.sh" >yum-update.log 2>&1; then
	    echo "update-yum-archive.sh failed; inspect yum-update.log"
	    exit 1
	fi
    fi
    if $debian; then
	if test -n "${DEBIAN_DIST}"; then
	    for dist in ${DEBIAN_DIST}; do
		ssh ${UPLOAD_HOST} "mkdir -p ${UPLOAD_PATH}/${PRODUCT}/debian/${dist}"
		if ! scp build/debian/${dist}/*_${VERSION}* ${UPLOAD_URI}/debian/${dist}; then
		    echo "Upload failed for $dist"
		    exit 1
		fi
	    done
	fi
	if test -n "${UBUNTU_DIST}"; then
	    for dist in ${UBUNTU_DIST}; do
		ssh ${UPLOAD_HOST} "mkdir -p ${UPLOAD_PATH}/${PRODUCT}/ubuntu/${dist}"
		if ! scp build/ubuntu/${dist}/*_${VERSION}* ${UPLOAD_URI}/ubuntu/${dist}; then
		    echo "Upload failed for $dist"
		    exit 1
		fi
	    done
	fi
	if ! ssh ${UPLOAD_HOST} "sudo -n /home/indexdata/proj/git-tools/update-archive/update-archive.sh --ubuntu=\"${UBUNTU_DIST}\" --debian=\"${DEBIAN_DIST}\""; then
	    echo "update-archive.sh failed"
	    exit 1
	fi
    fi
    echo "Upload successful - we hope"
    exit 0
fi

if $centos; then
    for dist in ${CENTOS_DIST}; do
	test -d build/centos/$dist && sudo rm -fr build/centos/$dist
	mkdir -p build/centos/$dist/rpmbuild
	mkdir -p build/centos/$dist/rpmbuild/BUILD
	mkdir -p build/centos/$dist/rpmbuild/BUILDROOT
	mkdir -p build/centos/$dist/rpmbuild/RPMS/noarch
	mkdir -p build/centos/$dist/rpmbuild/RPMS/x86_64
	mkdir -p build/centos/$dist/rpmbuild/SOURCES
	mkdir -p build/centos/$dist/rpmbuild/SPECS
	mkdir -p build/centos/$dist/rpmbuild/SRPMS
	cp ${TAR} build/centos/$dist/rpmbuild/SOURCES
	SPEC=${PRODUCT}.spec
	cp ${SPEC} build/centos/$dist/rpmbuild/SPECS
	cp IDMETA build/centos/$dist/rpmbuild
	cp ${ID_DEB_BUILD}/in-docker-rpm.sh build/centos/$dist
    done

fi

if $debian; then
    MKDEBSRC=${ID_DEB_BUILD}/id-mk-deb-src.sh
    if test ! -x $MKDEBSRC; then
        echo "$MKDEBSRC not found"
        exit 1
    fi
    if ! ${ID_DEB_BUILD}/id-mk-deb-changelog.sh; then
        echo "id-mk-deb-changelog.sh failed"
    fi
    rm -fr deb-src
    if ! ${MKDEBSRC}; then
        echo "id-mk-deb-src.sh failed"
    fi
    cd deb-src
    for f in *.dsc; do
        if test -f "$f"; then
            DSC=$f
        fi
    done
    cd ..
    if test -z "$DSC"; then
        echo "${DSC} does not exist"
        exit 1
    fi
fi
if test "${DEBIAN_DIST}"; then
    prepare_deb debian "${DEBIAN_DIST}"
fi
if test "${UBUNTU_DIST}"; then
    prepare_deb ubuntu "${UBUNTU_DIST}"
fi

if test "${CENTOS_DIST}"; then
    for dist in ${CENTOS_DIST}; do
	centos_map $dist
	D=$?
	F=$D
	test $F = "5" && F=5.5
	cd build/centos/$dist
	echo "FROM centos:$D" >Dockerfile
	echo "RUN yum install -y wget yum-utils redhat-rpm-config rpm-build gcc gcc-c++ make" >>Dockerfile
	echo "RUN rpm --import http://ftp.indexdata.com/pub/yum/centos/$F/RPM-GPG-KEY-indexdata" >>Dockerfile
	echo "RUN wget -q http://ftp.indexdata.com/pub/yum/centos/$F/indexdata.repo -P /etc/yum.repos.d/" >>Dockerfile
	echo 'ENTRYPOINT ["/bin/sh", "-c"]' >>Dockerfile
	docker build -t indexdata-build-centos:$D .
	echo -n "$dist "
	docker run --rm  -v "$PWD:/build" -w /build/rpmbuild indexdata-build-centos:$D ../in-docker-rpm.sh >log 2>&1
	if test $? -eq 0; then
		echo Done
	else
		echo "Fail. Inspect build/centos/$dist/log"
		exit 1
	fi
	cd ../../..
    done
fi
if test "${DEBIAN_DIST}"; then
    run_deb debian "${DEBIAN_DIST}"
fi
if test "${UBUNTU_DIST}"; then
    run_deb ubuntu "${UBUNTU_DIST}"
fi

# Local Variables:
# mode:shell-script
# sh-indentation: 4
# sh-basic-offset: 4
# End:

